package clientFunction;



import java.util.LinkedList;
import java.util.StringTokenizer;


/**
 * @author Tommaso
 * Classe che permette di dichiarare un oggetto access list e di collegarlo alle access list padre e figlie. 
 * Fornisce anche alcuni metodi utli per effettuare operazioni sull'oggetto ACL.
 */
public class ACL {

	/** ID identificativo della access list */
	private String ID;
	/** Lista degli ID degli utenti presenti nella access list*/
	private LinkedList<String> UserList = null;
	/** Lista delle access list padre*/
	private LinkedList<ACL> Ancestors = null;
	/** Lista delle access list figlie*/
	private LinkedList<ACL> Descendents = null;
	private int NumNodo;
	private LinkedList<Intervallo> Interval=null; 
	
	/**
	 * Costruttore della classe
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public ACL() {
		super();
		UserList = new LinkedList<String>();
		Ancestors = new LinkedList<ACL>();
		Descendents = new LinkedList<ACL>();
		Interval = new LinkedList<Intervallo>();
		
	}
	
	/**
	 * Costruttore della classe
	 * 
	 * @param	String	id	-	Parametro per inizializzare la variabile ID della classe
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public ACL(String id) {
		super();
		ID = id;
		UserList = new LinkedList<String>();
		Ancestors = new LinkedList<ACL>();
		Descendents = new LinkedList<ACL>();
		Interval = new LinkedList<Intervallo>();
	
	}
	
	/**
	 * Costruttore della classe
	 * 
	 * @param	String			id			-	Parametro per inizializzare la variabile ID della classe
	 * @param	List<String>	userlist	-	Lista degli utenti da inserire	
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public ACL(String id, LinkedList<String> userlist) {
		super();
		ID = id;
		UserList = new LinkedList<String>();
		addUsers(userlist);
		Ancestors = new LinkedList<ACL>();
		Descendents = new LinkedList<ACL>();
		Interval = new LinkedList<Intervallo>();
	}
	
	/**
	 * Metodo che setta l'ID della access List
	 * 
	 * @param	String id	-	Id da settare
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public void setID(String id) {
		ID = id;
	}
	
	public int getNum()
	{
		return this.NumNodo;
	}
	
	public void setNum(int n)
	{
		this.NumNodo=n;
	}
	
	public void setIntervallo(int min,int max)
	{
		Intervallo i=new Intervallo();
		
		i.setMinMax(min, max);
		
		Interval.add(i);
		
		SistemaIntervalli();
		
	}
	
	public void setIntervalli(LinkedList<Intervallo> list)
	{
		for(int i=0;i<list.size();++i)
		 	{ Interval.add(list.get(i));
		     
		     
		 	  SistemaIntervalli();}
		
	}
	
	public void copiaIntervalli(LinkedList<Intervallo> list)
	{
		Interval=new LinkedList<Intervallo>();
		for(int i=0;i<list.size();++i)
		  Interval.add(list.get(i));
		      
		    
		 	  
		
	}
	
	public int RicavaMax()
	{
		int max=0;
		
		for(int i=0;i<Interval.size();++i)
			if(Interval.get(i).getMax()>max)
				max=Interval.get(i).getMax();
		
		return max;
	}
	
	public LinkedList<Intervallo> getIntervalli()
	{
		return this.Interval;
	}
	
	public String CreaStringaIntervalli()
	{
		String s="";
		for(int i=0;i<this.getIntervalli().size();++i)
			{s+=this.getIntervalli().get(i).getMin()+","+this.getIntervalli().get(i).getMax();
			 if(i!=this.getIntervalli().size()-1)
				 s+="/";
			}
		return s;
		
	}
	
	private void SistemaIntervalli()
	{
		boolean fatto=false;
		LinkedList<Integer> vet=new LinkedList<Integer>();
		for(int j=0;j<Interval.size();++j)
			vet.add(0);
		
		
		do
		{	fatto=false;
			for(int i=0;i<Interval.size();++i)
				for(int j=0;j<Interval.size();++j)
					if((i!=j)&&(vet.get(i)==0)&&(vet.get(j)==0))
						{
						
						
						if(Interval.get(i).contains(Interval.get(j)))
							{
							 vet.set(j, 1);
							 fatto=true;
							}
						else
						 if(Interval.get(i).Consecutivo(Interval.get(j)))
						 {
						 
						 Interval.add(Intervallo.unisci(Interval.get(i), Interval.get(j)));
						 vet.add(0);
						 vet.set(i, 1);
						     vet.set(j, 1);
							 fatto=true;
						 }
						}
			}
			while(fatto==true);
	
				
		int cont=0;
		for(int j=0;j<vet.size();++j)
			if(vet.get(j)==1)
				{Interval.remove(j-cont);
				++cont;
				}
						
							
		
	}
	
	public int getEstremo(LinkedList<ACL> albero)
	{
		int min=Integer.MAX_VALUE;
		
		if(this.getDescendents().size()>0)
		for(int i=0;i<this.getDescendents().size();++i)
		{   
			ACL disc=new ACL();
			for(int j=0;j<albero.size();++j)
				if(this.getDescendents().get(i).getID().equals(albero.get(j).getID()))
			        disc=albero.get(j);
			
			if(disc.getNum()<min)
				min=disc.getIntervalli().get(0).getMin();
		}
		else
			min=this.NumNodo;
		
		return min;
	}
	
	/**
	 * Metodo che setta i predecessori della access List
	 * 
	 * @param	LinkedList<ACL>	ancestor	-	Lista predecessori
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public void setAncestor(LinkedList<ACL> ancestor){
		this.Ancestors = ancestor;
	}
	
	/**
	 * Metodo che setta i successori della access List
	 * 
	 * @param	LinkedList<ACL>	descendents	-	Lista successori
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public void setDescendents(LinkedList<ACL> descendents){
		this.Descendents = descendents;
	}

	/**
	 * Metodo che ritorna l'ID della access List
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	String	-	ID della access list
	 * 
	 */
	public String getID() {
		return ID;
	}

	/**
	 * Metodo che ritorna i predecessori della access List
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	LinkedList<ACL>	-	Lista predecessori
	 * 
	 */
	public LinkedList<ACL> getAncestor(){
		return this.Ancestors;
	}
	
	/**
	 * Metodo che ritorna i successori della access List
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	LinkedList<ACL>	-	Lista successori
	 * 
	 */
	public LinkedList<ACL> getDescendents(){
		return this.Descendents;
	}
	
	/**
	 * Metodo che aggiunge un utente alla access list
	 * 
	 * @param	String	user	-	Utente da aggiungere
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public boolean addUser(String user) {
				
		UserList.add(user);
	
	
		int[] vet=new int[UserList.size()];
		for(int i=0;i<UserList.size();++i)
			vet[i]=Integer.parseInt(UserList.get(i));

		
		for(int i=0;i<vet.length;++i)
			for(int j=0;j<vet.length-1;++j)
				if(vet[j]>vet[j+1])
				{
					int s=vet[j];
					vet[j]=vet[j+1];
					vet[j+1]=s;
				}
		UserList.clear();
		
		for(int i=0;i<vet.length;++i)
			UserList.add(Integer.toString(vet[i]));
	
	
		//aggiorno id
		String s="";
		
		for( int  i=0; i<UserList.size(); i++)
		  {if(i!=0)s=s+"-";
			s=s+UserList.get(i);
		  
		  }
		
		ID=s;
			
		return true;
	}
	
	/**
	 * Metodo che aggiunge una lista di utenti alla access list
	 * 
	 * @param	List<String>	users	-	Lista utenti da aggiungere
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public void addUsers(LinkedList<String> users) {
		for(int i=0; i<users.size(); i++)
			addUser(users.get(i));
	}

	/**
	 * Metodo che ritorna la dimensione della access list
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	int	-	numero di elementi nella access list
	 * 
	 */
	public int getSize() {
		
		return UserList.size();
	}

	/**
	 * Metodo che ritorna la lista degli utenti presenti nella access list
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	List<String>	-	lista degli utenti presenti nella access list
	 * 
	 */
	public LinkedList<String> getUserList() {
		return UserList;
	}

	/**
	 * Metodo che confronta le access list di due oggetti ACL
	 * 
	 * @param	ACL	toCheck	-	Oggetto con cui effettuare il confronto
	 * 
	 * @return	boolean		-	True	->	Le access list dei due oggetti son uguali
	 * 							False	->	Le access list dei due oggetti sono diverse
	 * 
	 */
	public boolean equals(ACL toCheck) {

		int a, i;
		if (toCheck.getSize() == getSize()) {
			for (i = 0; i < getSize(); i++) {
				for (a = 0; a < toCheck.getSize(); a++) {
					if (UserList.get(i).equals(toCheck.getUserList().get(a))) {
						break;
					}
				}
				if (a == toCheck.getSize())
					return false;
			}
		} else
			return false;
		return true;
	}

	/**
	 * Metodo che confronta le access list di due oggetti ACL per verificare che quella passata sia
	 *  contenuta in quella che invoca il metodo
	 * 
	 * @param	ACL	toCheck	-	Oggetto con cui effettuare il confronto
	 * 
	 * @return	boolean		-	True	->	La access list � contenuta in quella che invoca il metodo
	 * 							False	->	La access list non � contenuta in quella che invoca il metodo
	 * 
	 */
	public boolean contains(ACL toCheck) {
		int a, i;
		if (getSize() >= toCheck.getSize()) {
			for (i = 0; i < toCheck.getSize(); i++) {
				for (a = 0; a < getSize(); a++) {
					if (toCheck.getUserList().get(i).equals(UserList.get(a)))
						break;
				}
				if (a == getSize())
					return false;
			}
		} else
			return false;
		return true;
	}
	
	/**
	 * Metodo che calcola la differenza tra due oggetti ACL e restituisce la lista degli utenti 
	 * risultante dall'operazione, l'oggetto passato per parametro deve essere contenuto in quello 
	 * che invoca il metodo
	 * 
	 * @param	ACL	toCheck		-	Oggetto ACL da sottrarre
	 * 
	 * @return	List<String>	-	Lista degli utenti risultanti dalla differenza	
	 * 
	 */
	public LinkedList<String> sub(ACL	toSubtract){
		LinkedList<String> result = getUserList();
		for( int i = 0; i < toSubtract.getUserList().size();i++)
			for(int  a=0; a < result.size(); a++ ){
				if(result.get(a).equals(toSubtract.getUserList().get(i))){
					result.remove(a);
					break;
				}
			}
		return result;
	}
	
	/**
	 * Aggiungo un nuovo elemento padre all'oggetto ACL
	 * 
	 * @param	ACL		anc	-	ACL da inserire nella lista dei predecessori
	 * 
	 * @return	boolean		-	True	->	Inserimento avvenuto correttamente
	 * 
	 */
	public boolean addAncestor(ACL anc){
		Ancestors.add(anc);
		for(int i=0;i<Ancestors.size();i++)
			if(Ancestors.get(i).equals(anc))
				return true;
		return true;
	}
	
	/**
	 * Aggiungo un nuovo elemento figlio all'oggetto ACL
	 * 
	 * @param	ACL		anc	-	ACL da inserire nella lista dei successori
	 * 
	 * @return	boolean		-	True	->	Inserimento avvenuto correttamente
	 * 
	 */
	public boolean addDescendent(ACL desc){
		Descendents.add(desc);
		return true;
	}

}