package clientFunction;

import java.sql.ResultSet;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.StringTokenizer;





/**
 * @author Tommaso
 * Classe che permette di costruire il grafo degli accessi per un proprietario di file specificato 
 */
public class AccessGraphSEL {

	/** Lista delle access list presenti nel grafo degli accessi*/
	private LinkedList<ACLSEL> ACLList;
	/** Tipo: 0->BEL 1->SEL */
	int type;
	private LinkedList<ACLSEL> bottom;
	private LinkedList<ACLSEL> albero;

	/**
	 * Costruttore della classe
	 * 
	 * @param	int type  -  Tipo del livello, sel o bel
	 * 
	 * @return	ASSENTE
	 * 
	 */
	public AccessGraphSEL(int type){
		ACLList = new	LinkedList<ACLSEL>();
		this.type=type;
		this.bottom=new LinkedList<ACLSEL>();
		buildAccessGraph();
	}
	
	public void addACLBottom(ACLSEL acl)
	{
		bottom.add(acl);
	}
	
	public LinkedList<ACLSEL> getAllBottom()
	{
		return bottom;
	}
	
	
	/** 
	 * Aggiungo un acl alla lista 
	 * 
	 * @param	ACLSEL	newElement	-	Nuovo elemento da aggiungere
	 * 
	 * @return	ASSENTE
	 * 
	 * */
	public void addAcl(ACLSEL newElement) {
		LinkedList<String> ll=new LinkedList<String>();
		for(int i=0;i<ACLList.size();++i)
			ll.add(ACLList.get(i).getID());
		if(!ll.contains(newElement.getID()))
			ACLList.add(newElement);
	}
	
	/** 
	 * Aggiungo un acl alla lista 
	 * 
	 * @param	String			id			-	Id della access list
	 * @param	List<String>	userList	-	Lista degli utenti da inserire	
	 * 
	 * @return	ASSENTE
	 * 
	 * */
	public void addAcl(String id, LinkedList<String> userList) {
		ACLList.add(new ACLSEL(id,userList));
	}
	
	/** 
	 * Aggiungo un acl alla lista 
	 * 
	 * @param	String			id			-	Id della access list
	 * 
	 * @return	ASSENTE
	 * 
	 * */
	public void addAcl(String id) {
		ACLList.add(new ACLSEL(id));
	}

	/**
	 * Metodo che restituisce la lista di tutti gli acl
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	List<ACLSEL>	-	Riferimento alla lista di tutti gli acl
	 * 
	 */
	public LinkedList<ACLSEL> getAllAcl() {
		return ACLList;
	}
	
	/**
	 * Metodo che restituisce la lista di tutti gli acl con la dimensione specificata
	 * 
	 * @param	int	size	-	Dimensione degli acl
	 * 
	 * @return	List<ACL>	-	Lista di tutti gli acl con la dimensione specificata
	 * 
	 */
	public LinkedList<ACLSEL> getAclListBySize(int size){
		LinkedList<ACLSEL> acl = new LinkedList<ACLSEL>();
		
		
		if(size>0){
			for(int i=0; i < ACLList.size(); i++){
				if(ACLList.get(i).getSize()==size)
					acl.add(ACLList.get(i));
			}
		}	
		if(acl.size()==0)
			return null;	
		return acl;
	}	
	
	/**
	 * Controllo che esista un oggetto ACL nella lista con l'ID specificato
	 * 
	 * @param	String	ID	-	ID da cercare
	 * 
	 * @return	boolean		-	True	->	Esiste un oggetto ACL con l'id specificato
	 * 							False	->	Non esiste un oggetto ACL con l'id specificato
	 * 
	 */
	private boolean exist(String ID){
		for(int i = 0;i < ACLList.size(); i++){
			if(ACLList.get(i).getID().equals(ID))
				return true;}
		for(int i = 0;i < bottom.size(); i++){
				if(bottom.get(i).getID().equals(ID))
					return true;	
		}
		
		return false;
	}
	
	/**
	 * Metodo che restituisce l'oggeto ACL con l'ID specificato
	 * 
	 * @param	String	ID	-	ID da cercare
	 * 
	 * @return	ACL			-	Oggetto ACL trovato
	 * 
	 */
	private ACLSEL getAclById(String ID){
		for(int i = 0;i < ACLList.size(); i++){
			if(ACLList.get(i).getID().equals(ID))
				return ACLList.get(i);
		}
		return null;
	}
	
	
	
	/**
	 * Costruisce il grafo delle autorizzazioni 
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	AccessGraph	-	Lista delle ACL
	 * 
	 */
	public void buildAccessGraph() {		
		DBConnection connection = new DBConnection();
		try {
			Statement stmt = connection.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rset;	
			
			if(type==0) // BEL
				rset = stmt.executeQuery("SELECT groupidsource, groupiddestination from tokentable where level='BEL'");
			else        // SEL
				rset = stmt.executeQuery("SELECT groupidsource, groupiddestination from tokentable where level='SEL'");	
			
			
			ResultSet temp = rset;
		int cont=0;
			while (temp.next()){
			            ACLSEL acl1= new ACLSEL(temp.getString(1));
			            acl1.addUsers(getUserFromIdGroup(temp.getString(1)));
			            ACLSEL acl2= new ACLSEL(temp.getString(2));
			            LinkedList <String> supp=getUserFromIdGroup(temp.getString(2));
			            
			            if(supp.get(0).equals("ALL"))
			            {
			            		acl2.setall(true);
			            		for(int i=1;i<supp.size();++i)
			            			acl2.AddEsclusi(supp.get(i));
			            		acl2.setDescendents(new LinkedList<ACLSEL>());
			            		
			            }		
			            else
			            		acl2.addUsers(getUserFromIdGroup(temp.getString(2)));
			            ++cont;
			            
			            if (!exist(acl1.getID()))
			            {     
			            	 acl1.addDescendent(acl2);
			            	 
			            	 addAcl(acl1);
			    		}
			            
			            else
			            {
			            	for(int i = 0;i < ACLList.size(); i++){
			        			if(ACLList.get(i).getID().equals(acl1.getID()))
			        				{ACLList.get(i).addDescendent(acl2);
			        				 
			        				}
			            }}	
			            
			            if (!exist(acl2.getID()))
			            {     
			            	acl2.addAncestor(acl1);
			            	if(acl2.getall())
			            		addACLBottom(acl2);
			            	else
			            	    addAcl(acl2);
			    		}	
			            
			            else
			            {
			            	for(int i = 0;i < ACLList.size(); i++){
			        			if(ACLList.get(i).getID().equals(acl2.getID()))
			        				{ACLList.get(i).addAncestor(acl1);
			        				}}
			        		for(int i = 0;i < bottom.size(); i++){
				        			if(bottom.get(i).getID().equals(acl2.getID()))
				        				{bottom.get(i).addAncestor(acl1);
				        				}	
			        			
			            }}
			}
			
			
			
			rset.close();
			stmt.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		
		connection.closeConnection();
		
	}
	
	/**
	 * Aggiunge al grafo in modo ricorsivo tutte le access list ranggiungibili da quella specificata
	 * 
	 * @param	ACL 		src		-	access list di partenza
	 * @param	ResultSet	rset	-	Result set contenente tutti i token necessari a costruire il grafo
	 * 
	 * @return	ASSENTE
	 * 
	 */
	private void buildPath(ACLSEL src, ResultSet rset) {
		
		ACLSEL des = null;
		int pos;
		try {
			if (rset != null) {
				do {
					if (rset.getString(1).equals(src.getID())) {
						if (!exist(rset.getString(2))) {
							des = new ACLSEL(rset.getString(2), src.getUserList());
							addAcl(des);
						} else {
							des = getAclById(rset.getString(2));
							des.addUsers(src.getUserList());
						}
						src.addDescendent(des);		
						des.addAncestor(src);
						
						pos=rset.getRow();
						rset.first();
						buildPath(des, rset);
						rset.absolute(pos);
					}
				}while (rset.next());
			}
		} catch (Exception e) {
			System.err.println(e.toString());
		}

	}
	
	/**
	 * Estrae la lista degli user dall'id del gruppo
	 * 
	 * @param	String	ID	-	Id della chiave da cui estrarre l'id dell'utente
	 * 	
	 * @return	LinkedList<String> list		-	lista degli utenti estratti
	 * 
	 */
	private static LinkedList<String> getUserFromIdGroup(String ID) {
		StringTokenizer group=new StringTokenizer(ID,"-");
		LinkedList<String> list=new LinkedList<String>();
        while (group.hasMoreTokens())
        {   
        	list.add(group.nextToken());
        }
       
        
        return list;
	}


/**
 * Estrae ACL con la chiave corrispondente a quella passata
 * 
 * @param	String	key	                - 	chiave gruppo
 * 	
 * @return	ACL		                    -	ACL corrispondente alla chiave
 * 
 */
  public ACLSEL getACLfromID(String key)
   {
	  
	
	
	  
	  if(ACLList!=null)
	  for (int i=0;i<ACLList.size();++i)
		  if (key.equals(ACLList.get(i).getID()))
			  return ACLList.get(i);
	  
	  for (int i=0;i<bottom.size();++i)
		  if (key.equals(bottom.get(i).getID()))
			  return bottom.get(i);
	  
	  return null;
	  }
  
  
  public int getNmax()
	{
		int max=0;
		for (int i=0;i<ACLList.size();++i)
			if(ACLList.get(i).getSize()>max)
				max=ACLList.get(i).getSize();
		
		return max;
	}
  
  public ACLSEL getACLfromNum(int n)
  {
	  for (int i=0;i<albero.size();++i)
		  if (n==(albero.get(i).getNum()))
			  return albero.get(i);
	  return null;
	  }
  
  public int numera(String id, int n,LinkedList<ACLSEL> albero)
  {		
	  ACLSEL acl=new ACLSEL(); 
	  for(int i=0;i<albero.size();++i)
		  if(albero.get(i).getID().equals(id))
			  acl=albero.get(i);
			  
	  
	  if(acl.getDescendents().size()>0)
		  { 
		     for(int i=0;i<acl.getDescendents().size();++i)
			 {   
				     n=numera(acl.getDescendents().get(i).getID(),n,albero);
		    	 	
		    	 	
			
			     
			 }}
	
	  
	  acl.setNum(n);
	 
	  n++;
	  return n;
	  
  }
  
  public LinkedList<ACLSEL> TrasformaInAlbero()
  {
	  
	  
	  
	  
	  LinkedList<ACLSEL> albero=new LinkedList<ACLSEL>();
	  ACLSEL radice=new ACLSEL("radice");
	 
	  for(int i=0;i<this.getAclListBySize(2).size();++i)
		  radice.addDescendent(this.getAclListBySize(2).get(i));
	  LinkedList<String> giafatto=new LinkedList<String>();
	  
	
	  CreaAlbero(albero,giafatto,radice);
	 
	
	  for(int j=0;j<albero.size();++j)
		  if(albero.get(j).getID().equals("radice"))
			    albero.remove(j);
			  
	  ACLSEL rad=new ACLSEL("radice");
	  for(int i=0;i<this.getAclListBySize(2).size();++i)
		  rad.addDescendent(this.getAclListBySize(2).get(i));
	  albero.add(rad);
	
	 
	  this.numera("radice", 1, albero);
	 
	  
	  for(int i=0;i<albero.size();++i)
		  for(int j=0;j<albero.size()-1;++j)
		  {
			  ACLSEL supp=new ACLSEL();
			  if(albero.get(j).getNum()>albero.get(j+1).getNum())
			  {
				  supp=albero.get(j);
				  albero.set(j, albero.get(j+1));
				  albero.set(j+1, supp);
			  }
		  }
	  
	  
	  
	  this.albero=albero;
	  return albero;
  }
  
  
  private boolean ACLcontenutoInAlbero(LinkedList<ACLSEL> list,ACLSEL acl)
  {
	  for(int i=0;i<list.size();++i)
		  if(list.get(i).getID().equals(acl.getID()))
			  return true;
	  return false; 
  }
  
  private ACLSEL getACLdaAlbero(LinkedList<ACLSEL> albero,String id)
  {
	  for(int i=0;i<albero.size();++i)
		  if(albero.get(i).getID().equals(id))
			  return albero.get(i);
	  return null;
  }
  
  public void CreaAlbero(LinkedList<ACLSEL> albero,LinkedList<String> giafatto,ACLSEL acl)
  {
	 if(!acl.getID().equals("radice"))
	    {
		 acl=this.getACLfromID(acl.getID());
	    }
	 if(acl.getDescendents().size()==0)
	 {	
		  albero.add(new ACLSEL(acl.getID()));}
	 else	 
	 {   ACLSEL nuovo=null;
		 if(!this.ACLcontenutoInAlbero(albero, acl)){
		     nuovo=new ACLSEL(acl.getID());
		 }
		 else
			 {nuovo=this.getACLdaAlbero(albero, acl.getID());
			 }
		 for(int i=0;i<acl.getDescendents().size();++i)
	 {  
		 
		  
		
		 boolean t=false;
		 ACLSEL figlio=acl.getDescendents().get(i);
		
		 if(!giafatto.contains(figlio.getID()))
			 {    nuovo.addDescendent(new ACLSEL(figlio.getID()));
			      giafatto.add(figlio.getID());
				 t=true;
			 }
		 if(i==acl.getDescendents().size()-1)
		 {
			 albero.add(nuovo);
			
		 }	 
			if(t==true)
				 {CreaAlbero(albero,giafatto,acl.getDescendents().get(i));
				 t=true;
				 
				 }
			
		 }}
	 
	 
		  
	  
	  return;
  

}
  
  public LinkedList<ACLSEL> getAlbero() {
		return albero;
	}
  
public void CreaIntervalli()
{
	this.buildAccessGraph();
	
	
	
	
	this.TrasformaInAlbero();
	
	
	for(int i=1;i<albero.size()+1;++i)
	   this.getACLfromNum(i).setIntervallo(this.getACLfromNum(i).getEstremo(this.getAlbero()), i);
	    
	 
	for(int i=0;i<albero.size();++i)
		if(!albero.get(i).getID().equals("radice"))
			{this.getACLfromID(albero.get(i).getID()).setIntervallo(albero.get(i).getIntervalli().get(0).getMin(), albero.get(i).getIntervalli().get(0).getMax());
			 this.getACLfromID(albero.get(i).getID()).setNum(albero.get(i).getNum());
			}
	
	
	

	
	
	
	for(int j=0;j<this.bottom.size();++j)
		for(int t=0;t<this.bottom.get(j).getAncestor().size();++t)
			this.getACLfromID(this.bottom.get(j).getAncestor().get(t).getID()).setIntervalli(this.bottom.get(j).getIntervalli());
	
	
	
	for(int i=this.getNmax();i>2;--i)
	{   
		LinkedList<ACLSEL> list=this.getAclListBySize(i);
		if(list!=null)
		for(int j=0;j<list.size();++j)
			for(int t=0;t<list.get(j).getAncestor().size();++t)
				this.getACLfromID(list.get(j).getAncestor().get(t).getID()).setIntervalli(list.get(j).getIntervalli());
			
	}
	
		
	SalvaIntervalliInDB();
	
	
	
}

	public void UpdateToken(DBConnection connection,String s,String tokenID)
	{
		try {
			Statement stmt = connection.getConnection().createStatement();

			stmt.execute("Update tokentable set interval='"+s+ "' where tokenid = '"+tokenID+"'");
			
			stmt.close();

		} catch (Exception e) {
			System.err.println(e.toString());
			
		}
		
	}

	public void SalvaIntervalliInDB()
	{
		DBConnection connection = new DBConnection();
		try {
			Statement stmt = connection.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rset;	
			rset = stmt.executeQuery("SELECT groupidsource,groupiddestination,tokenid from tokentable where level='SEL'");
			ResultSet temp = rset;
		
			while (temp.next()){
				    ACLSEL aclsource=this.getACLfromID(temp.getString(1));
				    ACLSEL acldest=this.getACLfromID(temp.getString(2));
				    if(aclsource.getSize()==2)
			    	   UpdateToken(connection,acldest.CreaStringaIntervalli()+"*"+aclsource.CreaStringaIntervalli(),temp.getString(3)); 
				    else
			           UpdateToken(connection,acldest.CreaStringaIntervalli(),temp.getString(3));
			
			}
			
			
			
			rset.close();
			stmt.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		
		connection.closeConnection();
		return;
	}
	
	public void CaricaIntervalli()
	{
		DBConnection connection = new DBConnection();
		try {
			Statement stmt = connection.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rset;	
			rset = stmt.executeQuery("SELECT groupidsource,groupiddestination,interval from tokentable where level='SEL'");
			ResultSet temp = rset;
		
			while (temp.next()){
				    ACLSEL aclsource=this.getACLfromID(temp.getString(1));
				    ACLSEL acldest=this.getACLfromID(temp.getString(2));
				    String inter=temp.getString(3);
				    
				    StringTokenizer group=new StringTokenizer(inter,"*");
					LinkedList<String> list=new LinkedList<String>();
				
					
					if(group!=null)
			        while (group.hasMoreTokens())
			        {
			        	list.add(group.nextToken());
			        }
			        
			        for(int i=0;i<list.size();++i)
			        {
			        	StringTokenizer group2=new StringTokenizer(list.get(i),"/"); 
			        	LinkedList<String> intervalli=new LinkedList<String>();
				        while (group2.hasMoreTokens())
				        {
				        	intervalli.add(group2.nextToken());
				        }
				        if(i==0)
				        {
				        	LinkedList<Intervallo> interv=RicavaIntervalli(intervalli);
				        	acldest.copiaIntervalli(interv);
				        	acldest.setNum(acldest.RicavaMax());
				        }
				        if(i==1)
				        {
				        	LinkedList<Intervallo> interv=RicavaIntervalli(intervalli);
				        	aclsource.copiaIntervalli(interv);
				        	aclsource.setNum(aclsource.RicavaMax());
				        }
			        }
			
			}
			
			
			
			rset.close();
			stmt.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		
		connection.closeConnection();
		return;
	}
	
	
	public LinkedList<Intervallo> RicavaIntervalli(LinkedList<String> i)
	{
		LinkedList<Intervallo> inter=new LinkedList<Intervallo>();
		
		for(int j=0;j<i.size();++j)
		{
		StringTokenizer group2=new StringTokenizer(i.get(j),","); 
		LinkedList<String> valori=new LinkedList<String>();
        while (group2.hasMoreTokens())
        {
        	valori.add(group2.nextToken());
        }
        Intervallo supp=new Intervallo(Integer.parseInt(valori.get(0)),Integer.parseInt(valori.get(1)));
        inter.add(supp);
		}
		
		return inter;
	}



}

