package clientFunction;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

/**
 * @author Tommaso
 * Classe che fornisce metodi statici per criptare e decriptare file e per creare una creare una chiave da 
 * utilizzare con un algoritmo crittografico  
 */
public class CryptoFunction {
        
	/** Dimensione array di byte letto da file da criptare */
	public static int BLOCK_SIZE = 4095;
	/** Dimensione array di byte da decriptare */
	public static int AES_BLOCK_SIZE = ((BLOCK_SIZE / 16) + 1) * 16;
	/** Algoritmo utilizzato per crittografare il file */
	public static String CRIPTOGRAPHIC_ALGORITHM = "AES";
	
	/**
	 * Metodo che cripta un file con algoritmo AES con chiave a 128 bit passata
	 * come parametro e lo salva su hard disk
	 * 
	 * @param	byte[]	secretKey		-	Vettore di byte contenente la chiave da usare per criptare il file
	 * @param	String	filePath		-	Path del file
	 * @param	String	saveDirectory	-	Directory dove salvare il file
	 * @param	boolean hash			-	True	->	La chiave va sottoposta alla funzione di hash
	 * 										False	->	La chiave non va sottoposta alla funzione di hash
	 * 
	 * @return	ASSENTE
	 */
	public static String encryptFile(byte[] secretKey, String filePath, String saveDirectory,boolean hash) {

		try {
			byte[] skey;
			byte[] encrypted = null;
			byte[] finalBlock;
			byte[] original;
 
			FileInputStream fin;
			BufferedInputStream dis;
			FileOutputStream fout;
			BufferedOutputStream dos;
			
			if(hash)
			{
				skey=hashSecretKey(secretKey);
			}
			else 
			{
				skey=secretKey;
			}	
			
			SecretKeySpec skeySpec = new SecretKeySpec(skey, CRIPTOGRAPHIC_ALGORITHM);
			Cipher cipher = Cipher.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			cipher.init(Cipher.ENCRYPT_MODE, skeySpec);
			
			fin = new FileInputStream(filePath);
			dis = new BufferedInputStream(fin);
			fout = new FileOutputStream(saveDirectory);
			dos = new BufferedOutputStream(fout);
			int numByte;
			original = new byte[BLOCK_SIZE];
			while (dis.available() != 0) {
				numByte = dis.read(original);
				if (numByte < AES_BLOCK_SIZE) {
					finalBlock = new byte[numByte];
					//finalBlock = Arrays.copyOf(original, numByte);
                                        finalBlock = arrayCopy(original, numByte);
					encrypted = cipher.doFinal(finalBlock);
				} else
					encrypted = cipher.doFinal(original);
				dos.write(encrypted);
			}
			dos.flush();
			fout.close();
			fin.close();
		}
		// Catches any error conditions
		catch (Exception e) {
			System.err.println(e.toString()+" - Mentre cripto il file");
			return e.toString()+" - Mentre cripto il file";
		}
		return "CRYPT OK";
	}
	
	/**
	 * Metodo che decripta un file con algoritmo AES con chiave a 128 bit passata
	 * come parametro e lo salva su hard disk
	 * 
	 * @param	byte[]	secretKey		-	Vettore di byte contenente la chiave da usare per decriptare il file
	 * @param	String	filePath		-	Path del file
	 * @param	String	saveDirectory	-	Directory dove salvare il file
	 * @param	boolean hash			-	True	->	La chiave va sottoposta alla funzione di hash
	 * 										False	->	La chiave non va sottoposta alla funzione di hash
	 * 
	 * @return	ASSENTE
	 */
	public static String decryptFile(byte[] secretKey, String filePath, String saveDirectory, boolean hash) {
		try {
			byte[] skey;
			byte[] encrypted = null;
			byte[] finalBlock;
			byte[] original;
			int numByte;
			FileInputStream fin;
			BufferedInputStream dis;
			FileOutputStream fout;
			BufferedOutputStream dos;
			fin = new FileInputStream(filePath);
			dis = new BufferedInputStream(fin);
			fout = new FileOutputStream(saveDirectory);
			dos = new BufferedOutputStream(fout);			
			if(hash)
			{
				skey=hashSecretKey(secretKey);
			}
			else 
			{
				skey=secretKey;
			}
			SecretKeySpec skeySpec = new SecretKeySpec(skey, CRIPTOGRAPHIC_ALGORITHM);
			Cipher cipher = Cipher.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			cipher.init(Cipher.DECRYPT_MODE, skeySpec);
			encrypted = new byte[AES_BLOCK_SIZE];
			while (dis.available() != 0) {
				numByte = dis.read(encrypted);
				if (numByte < AES_BLOCK_SIZE) {
					finalBlock = new byte[numByte];
					//finalBlock = Arrays.copyOf(encrypted, numByte);
                                        finalBlock = arrayCopy(encrypted, numByte);
					original = cipher.doFinal(finalBlock);
				} else
					original = cipher.doFinal(encrypted);
				dos.write(original);
			}
			dos.flush();
			fout.close();
			fin.close();
		}
		// Catches any error conditions
		catch(FileNotFoundException fnfe){
			System.err.println(fnfe.toString()+" - Mentre decripto il file");
			return fnfe.toString()+ " - Mentre decripto il file";
		}
		catch(Exception e) {
			System.err.println(e.toString()+" - Mentre decripto il file");
			return e.toString()+" - Mentre decripto il file";
		}
		return "DECRYPT OK";
	}
	
	/**
	 * Funzione che decripta un array di byte
	 * 
	 * @param	byte[]	encrypted	-	Array di byte da decriptare
	 * @param	byte[]	AESKey		-	Chiave con la quale decriptare, chiave "AES" a 128bit(16 byte)
	 * 
	 * @return	byte[]				-	Array di byte decriptato
	 * 
	 */
	public static byte[] decrypt(byte[] encrypted, byte[] AESKey) {

		byte[] decrypted = null;
		try {
			SecretKeySpec skeySpec = new SecretKeySpec(AESKey, CRIPTOGRAPHIC_ALGORITHM);
			Cipher cipher = Cipher.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			cipher.init(Cipher.DECRYPT_MODE, skeySpec);
			decrypted = cipher.doFinal(encrypted);
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		return decrypted;
	}
	
	/**
	 * Metodo che crea e restituisce una chiave da 128bit da utilizzare per l'algoritmo di crittografia
	 * 
	 * @param	ASSENTE
	 * 
	 * @return	byte[]	-	Array di byte contenente la chiave generata
	 * 
	 */
	public static byte[] createKey() {
		byte[] keyAES = null;
		try {
			KeyGenerator kgen = KeyGenerator.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			kgen.init(128);
			SecretKey skey = kgen.generateKey();
			keyAES = skey.getEncoded();
		} catch (Exception e) {
			System.err.println(e.toString()+" - Durante la creazione della chiave" +
					" per l'algoritmo di crittografia AES");
		}
		return keyAES;
	}
	
	/**
	 * Metodo che riceve come parametro il path del file e restituisce solo il nome del file
	 * 
	 * @param	String	path	-	Path del fiel	
	 * 
	 * @return	String			-	Nome del file estratto dal path
	 */
	private static String getFileName(String path) {
		StringBuffer fileName = new StringBuffer();
		for (int c = (path.length() - 1); (c != 0 && path.charAt(c) != File.separatorChar); c--)
			fileName.append(path.charAt(c));
		fileName.reverse();
		return fileName.toString();
	}
	
	/** 
	 * Metodo che applica la funzione di has alla chiave
	 */
	
	public static byte[] hashSecretKey(byte[] secretKey)
	{
		byte[] hashedKey = new byte[16];
		try{
			SecretKeySpec skeySpec = new SecretKeySpec(secretKey, CRIPTOGRAPHIC_ALGORITHM);
			Mac mac = Mac.getInstance("HmacSHA1");
			mac.init(skeySpec);
			byte[] temp = mac.doFinal();
			for(int i=0;i<16;i++)
			hashedKey[i]=temp[i];
		}catch(Exception e)
		{
			System.err.println(e.toString());
		}
		return hashedKey;
	}
        
	/** 
	 * Metodo per copiare l'array (per java 1.5)
	 */
	
        private static byte[] arrayCopy(byte[] original, int length) {
            byte[] copy = new byte[length];
            System.arraycopy(original, 0, copy, 0, length);
            return copy;
        }


}
