package clientFunction;


import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.math.BigInteger;
import java.security.KeyPair;
import java.security.KeyPairGenerator;
import java.sql.Date;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.StringTokenizer;
import java.util.Vector;


import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;
import javax.crypto.spec.DHParameterSpec;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

public class User {
	
	private Integer id = -1;
	private String name;
	private String surname;
	private String stringBirthDate;
	private java.sql.Date birthDate;
	private String birthPlace;
	private String nationality;
	private String maritalStatus;
	private String username;
	private String password = "";
	private String confirmPassword = "";
	private Boolean logged = false;

	
	public User() {
	
	}

	/**
	 * 
	 * @param name
	 * @param surname
	 * @param birthDate
	 * @param birthPlace
	 * @param nationality
	 * @param maritalStatus
	 * @param username
	 * @param password
	 * @param confirmPassword
	 */
	/*public User(String name, String surname, Date birthDate, String birthPlace,
			String nationality, String maritalStatus, String username, 
			char[] password, char[] confirmPassword) {
	
		setName(name);
		setSurname(surname);
		setBirthDate(birthDate);
		setBirthPlace(birthPlace);
		setNationality(nationality);
		setMaritalStatus(maritalStatus);
		setUsername(username);
		setPassword(password);
		setConfirmPassword(confirmPassword);
	}*/
	
	public void setUser(String name, String surname, String birthDate, String birthPlace,
			String nationality, String maritalStatus, String username, 
			String password, String confirmPassword) {
		
		setName(name);
		setSurname(surname);
		setBirthDate(birthDate);
		setBirthPlace(birthPlace);
		setNationality(nationality);
		setMaritalStatus(maritalStatus);
		setUsername(username);
		setPassword(password);
		setConfirmPassword(confirmPassword);
	}
	
	
	/**
	 * Permette di registrare l'utente
	 * @throws Exception
	 */
	public final void register() throws Exception{
		//createKey();
		save();
		//addDHKPair();
	}
	
	
	public boolean login(String username, String password) throws Exception{
		
		String sql = "SELECT  subjectid, name, username, password from usertable where username = '%s'";
		sql = String.format(sql, username);

		ResultSet rset = Configuration.getConnection().createStatement().executeQuery(sql);
		
		// Utente non presente
		if (rset.next()){
			
			if ((Utility.toStr(Utility.SHA1(password))).equals(Utility.toStr(rset.getBytes("password")))) {
				setId(rset.getInt("subjectid"));
				setUsername(rset.getString("username"));
				setName(rset.getString("name"));
				setLogged(true);	
				return true;
			} else {
				return false;
			}

		} else{	
			return false;
		}
	
	
	}
	
	public boolean login() throws Exception{
		return this.login(getUsername(), getPassword());
	}
	
	
	/**
	 * Recupera la chiave dell'utente.
	 * @return user key
	 * @throws Exception in caso di fallito recupero
	 */
	public byte[] getKey() throws Exception{
		return Utility.getKey(getUsername());		
	}

	/**
	 * Salva l'utente nel databse
	 * @throws Exception in caso di errore nel db
	 */
	public void save() throws Exception {
		
		check();
	
		Statement stmt = Configuration.getConnection().createStatement(ResultSet.TYPE_SCROLL_SENSITIVE,ResultSet.CONCUR_UPDATABLE);
		
		ResultSet r = stmt.executeQuery("select * from usertable"); 
		
		r.moveToInsertRow();
		
		r.updateString("name", name);
		r.updateString("surname", surname);
		r.updateDate("birthdate", birthDate);
		r.updateString("birthplace", birthPlace);
		r.updateString("nationality", nationality);
		r.updateString("maritalstatus", maritalStatus);
		r.updateString("username", username);		
		r.updateBytes("password", Utility.SHA1(password));
		
		r.insertRow();
		stmt.close();
		
		
	}
	
	/**
	 * Valida i campi dell'utente prima di essere inseriti nel database
	 * @throws Exception in caso di errori sintattici/semantici
	 */
	public void check() throws Exception {

		String errors = "";
		
		if (getName().length() < 4)
			errors += "- Name must have at least 4 characters\n";
		
		if (getSurname().length() < 4)
			errors += "- Surname must have at least 4 characters\n";
		
		if(Utility.stringToDate(getStringBirthDate()) == null) {
			errors += "- Birth date is not a date\n";
		} else {
			setBirthDate(Utility.stringToDate(getStringBirthDate()));
		}
	        
	    if (getBirthPlace().length() < 4) 
	    	errors += "- Birth Place must have at least 4 characters\n";
	       
	    if (getNationality().length() < 2) 
	    	errors += "- Nationality must have at least 2 characters\n";
	       
	    if (getMaritalStatus().length() == 0) 
	    	errors += "- Marital Status must be selected\n";
	    
	    if(getUsername().length() < 4) 
	    	errors += "- Username must have at least 8 characters\n";
	     
	    if(getPassword().length() < 4 || getConfirmPassword().length() < 4) 
	    	errors += "- Password must have at least 4 characters\n";
	    
	    if(!getPassword().equals(getConfirmPassword()))
	    	errors += "- Passwords do not match\n";
	
		if(errors.length() > 0)
			throw new Exception(errors);
			
	}
	
	
	/**
	 * Carica tutti i gruppi associati all'utente
	 * @return Vettore con tutti i gruppi associati
	 * @throws Exception
	 */
	public Vector<String>  loadGroups() throws Exception{
		Vector<String> vector = new Vector<String>();
		
		String sql = "SELECT name from grouptable where owner = '%s';";
		sql = String.format(sql, getId());
		
		ResultSet rset = Configuration.getConnection().createStatement().executeQuery(sql);

		if (rset != null) {
				while (rset.next()) {
					
					vector.add(rset.getString("name"));
				 
				}
			rset.close();
		}
		
		return vector;
	}
	
	public static String prova() throws JSONException {
		 JSONObject ob2 = new JSONObject();
		 ob2.put("pwd", "minchia");
		 ob2.put("nome", "Flavio");
		 
		 JSONObject ob1 = new JSONObject();
		 ob1.put("pwd", "aSFASDV");
		 ob1.put("nome", "CIVE");
		 
		 JSONArray array = new JSONArray();
		 array.put(ob1);
		 array.put(ob2);
		 
		 JSONObject finale = new JSONObject();
		 finale.put("values", array);
		  
		 return finale.toString();
	}
	
	/**
	 * Recupera la chiave dell'utente.
	 * @return user key
	 * @throws Exception in caso di fallito recupero
	 */
	public JSONObject getPinInfo() throws Exception{
		JSONObject finale = new JSONObject();

		String sql = "SELECT  subjectid, name, surname, hashpwd from usertable where subjectid = '%s'";
		sql = String.format(sql, this.getId());

		ResultSet rset = Configuration.getConnection().createStatement().executeQuery(sql);
		
		// Utente non presente
		if (rset.next()){
			JSONObject ob2 = new JSONObject();
			ob2.put("hashpin", rset.getString("hashpwd"));
			ob2.put("info", rset.getString("subjectid") + rset.getString("name") + rset.getString("surname"));
		
			JSONArray array = new JSONArray();
			array.put(ob2);
			 
			finale.put("values", array);
		
		}
		return finale;
	}
	
	public void insertDHandHashInDB(String json) throws SQLException, Exception {
		JSONObject root = new JSONObject(json);

			String sql = "INSERT INTO dhkatable (subjectid,publickey,privatekey) VALUES('%d', '%s', '%s');";
			sql = String.format(sql, getId(), 
					root.getString("public"),
					root.getString("private")
					);
			
			Configuration.getConnection().createStatement().execute(sql);
			
			sql = "UPDATE usertable SET hashpwd = '%s' WHERE subjectid = '%d';";
			sql = String.format(sql,
					root.getString("hash"),
					getId()
					);
			
			Configuration.getConnection().createStatement().execute(sql);
			
	}
	
	/* restituisce l'elenco degli utenti escluso il proprietario
	 * per la condivisione dei file
	 */
	public JSONObject getUsersList() throws JSONException {

		JSONObject response = new JSONObject();
		JSONArray array = new JSONArray();
		
		try {
			Statement stmt = Configuration.getConnection().createStatement();
			ResultSet rset  = stmt
					.executeQuery("SELECT subjectid,name,surname from usertable where subjectid <> " + getId() + " AND subjectid <> 0 order by surname, name ");

			if (rset != null) {
				while (rset.next()) {
						JSONObject temp = new JSONObject();
						temp.put("id", rset.getString(1));
						temp.put("name", rset.getString(2));
						temp.put("surname", rset.getString(3));
						array.put(temp);
					}

			}

			rset.close();
			stmt.close();

		} catch (Exception e) {
			System.err.println(e.toString());
		}

		response.put("values", array);
		return response;
	}
	
	
	public static void main(String args[]) {
		try {
			
			
			SELSharedFile a = new SELSharedFile();
						
			String users = "38,39,42,43";
			String[] userList = users.split(",");
			
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	

	/**
	 * Controlla l'esistenza dell'utente nel database
	 * @return true in caso di successo, false in caso di insuccesso
	 * @throws Exception
	 */
	boolean isStored() throws Exception{
		return getSubjecID() != -1 ? true : false;
	}
	
	/**
	 * Ottiene l'id dell'utente
	 * @return ID user, -1 in caso di errore
	 * @throws Exception
	 */
	public Integer getSubjecID() throws Exception {

		Integer subjectid  = getId();
		
		// Se non ho ancora letto dal db il codice getId = -1
		if( subjectid == -1){
						
			String sql = "SELECT subjectid from usertable where username='%s'";
			sql = String.format(sql, username);
	
			ResultSet rset = Configuration.getConnection().createStatement().executeQuery(sql);
			
			if (rset != null) {
				rset.next();
				subjectid = rset.getInt("subjectid");
			} else {
				subjectid = -1;
			}
			rset.close();
		}
		return subjectid;
	}


	/**
	 * Memorizza la chiave privata e pubblica nel database
	 * @throws Exception
	 */
	public void addDHKPair() throws Exception {

		KeyPair KPair = generateKPair();
		Integer id = getSubjecID();
		
		if (id == -1)
			throw new Exception("User not exists!");

		String sql = "INSERT INTO dhkatable (subjectid,publickey,privatekey) VALUES('%d', '%s', '%s');";
		sql = String.format(sql, id, 
				Utility.toStr(KPair.getPublic().getEncoded()),
				Utility.toStr(Utility.encrypt(KPair.getPrivate().getEncoded(), getKey()))
				);
		
		Configuration.getConnection().createStatement().execute(sql);
	}

	private KeyPair generateKPair() throws Exception {
		KeyPair KPair = null;
		DHParameterSpec dhSkipParamSpec;
		dhSkipParamSpec = new DHParameterSpec(skip1024Modulus, skip1024Base);

		KeyPairGenerator KpairGen = KeyPairGenerator.getInstance("DH");
		KpairGen.initialize(dhSkipParamSpec);
		KPair = KpairGen.generateKeyPair();

		return KPair;
	}
	
	/** The 1024 bit Diffie-Hellman modulus values used by SKIP */
	private static final byte skip1024ModulusBytes[] = {(byte) 0xF4, (byte) 0x88, (byte) 0xFD,
			(byte) 0x58, (byte) 0x4E, (byte) 0x49, (byte) 0xDB, (byte) 0xCD, (byte) 0x20,
			(byte) 0xB4, (byte) 0x9D, (byte) 0xE4, (byte) 0x91, (byte) 0x07, (byte) 0x36,
			(byte) 0x6B, (byte) 0x33, (byte) 0x6C, (byte) 0x38, (byte) 0x0D, (byte) 0x45,
			(byte) 0x1D, (byte) 0x0F, (byte) 0x7C, (byte) 0x88, (byte) 0xB3, (byte) 0x1C,
			(byte) 0x7C, (byte) 0x5B, (byte) 0x2D, (byte) 0x8E, (byte) 0xF6, (byte) 0xF3,
			(byte) 0xC9, (byte) 0x23, (byte) 0xC0, (byte) 0x43, (byte) 0xF0, (byte) 0xA5,
			(byte) 0x5B, (byte) 0x18, (byte) 0x8D, (byte) 0x8E, (byte) 0xBB, (byte) 0x55,
			(byte) 0x8C, (byte) 0xB8, (byte) 0x5D, (byte) 0x38, (byte) 0xD3, (byte) 0x34,
			(byte) 0xFD, (byte) 0x7C, (byte) 0x17, (byte) 0x57, (byte) 0x43, (byte) 0xA3,
			(byte) 0x1D, (byte) 0x18, (byte) 0x6C, (byte) 0xDE, (byte) 0x33, (byte) 0x21,
			(byte) 0x2C, (byte) 0xB5, (byte) 0x2A, (byte) 0xFF, (byte) 0x3C, (byte) 0xE1,
			(byte) 0xB1, (byte) 0x29, (byte) 0x40, (byte) 0x18, (byte) 0x11, (byte) 0x8D,
			(byte) 0x7C, (byte) 0x84, (byte) 0xA7, (byte) 0x0A, (byte) 0x72, (byte) 0xD6,
			(byte) 0x86, (byte) 0xC4, (byte) 0x03, (byte) 0x19, (byte) 0xC8, (byte) 0x07,
			(byte) 0x29, (byte) 0x7A, (byte) 0xCA, (byte) 0x95, (byte) 0x0C, (byte) 0xD9,
			(byte) 0x96, (byte) 0x9F, (byte) 0xAB, (byte) 0xD0, (byte) 0x0A, (byte) 0x50,
			(byte) 0x9B, (byte) 0x02, (byte) 0x46, (byte) 0xD3, (byte) 0x08, (byte) 0x3D,
			(byte) 0x66, (byte) 0xA4, (byte) 0x5D, (byte) 0x41, (byte) 0x9F, (byte) 0x9C,
			(byte) 0x7C, (byte) 0xBD, (byte) 0x89, (byte) 0x4B, (byte) 0x22, (byte) 0x19,
			(byte) 0x26, (byte) 0xBA, (byte) 0xAB, (byte) 0xA2, (byte) 0x5E, (byte) 0xC3,
			(byte) 0x55, (byte) 0xE9, (byte) 0x2F, (byte) 0x78, (byte) 0xC7};

	/** The SKIP 1024 bit modulus */
	private static final BigInteger skip1024Modulus = new BigInteger(1, skip1024ModulusBytes);

	/** The base used with the SKIP 1024 bit modulus */
	private static final BigInteger skip1024Base = BigInteger.valueOf(2);

	public Date getBirthDate() {
		return birthDate;
	}
	
	public String getStringBirthDate() {
		return stringBirthDate;
	}

	public void setBirthDate(java.sql.Date date) {
		this.birthDate = date;
	}
	
	public void setBirthDate(String birthDate) {
		this.stringBirthDate = birthDate;
	}

	public String getBirthPlace() {
		return birthPlace;
	}

	public void setBirthPlace(String birthPlace) {
		this.birthPlace = birthPlace;
	}

	public String getMaritalStatus() {
		return maritalStatus;
	}

	public void setMaritalStatus(String maritalStatus) {
		this.maritalStatus = maritalStatus;
	}

	public String getName() {
		return name;
	}
	
	public boolean hasKeys() throws Exception {
		Integer subjectid  = getId();
		Boolean hasKey = false;
		
		// Se non ho ancora letto dal db il codice getId = -1
		if( subjectid != -1){
						
			String sql = "SELECT hashpwd from usertable where subjectid=" + subjectid;
			//sql = String.format(sql, subjectid);

			ResultSet rset = Configuration.getConnection().createStatement().executeQuery(sql);
			
			if (rset != null) {
				rset.next();
				
				//int hash = rset.getString("hashpwd").length();
				if (rset.getString("hashpwd") != null){
					hasKey = true;
				}
			}
			rset.close();
		}
		return hasKey;
		//return true;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getNationality() {
		return nationality;
	}

	public void setNationality(String nationality) {
		this.nationality = nationality;
	}

	public String getPassword() {
		return password;
	}

	public void setPassword(String password) {
		this.password = password;
	}

	public String getSurname() {
		return surname;
	}

	public void setSurname(String surname) {
		this.surname = surname;
	}

	public String getUsername() {
		return username;
	}

	public void setUsername(String username) {
		this.username = username;
	}

	public String getConfirmPassword() {
		return confirmPassword;
	}

	public void setConfirmPassword(String confirmPassword) {
		this.confirmPassword = confirmPassword;
	}

	public Integer getId() {
		return this.id;
	}

	public void setId(Integer id) {
		this.id = id;
	}
	
	public void setLogged(Boolean logged) {
		this.logged = logged;
	}

	public Boolean getLogged() {
		return logged;
	}
	
}

