#ifndef DHJSON_H_INCLUDED
#include "dhjson.h"

#include "json_spirit.h"
#include <cassert>
#include <algorithm>
#include <fstream>
#include <boost/bind.hpp>

using namespace std;
using namespace boost;
using namespace json_spirit;


//-----------------------------------------------------------------------------
void add_pair( Object& obj, const string& name, const Value& value )
{
    obj.push_back( Pair( name, value ) );
}

//-----------------------------------------------------------------------------
Object& add_child_obj( Object& parent, const string& name )
{
    parent.push_back( Pair( name, Object() ) );

    return parent.back().value_.get_obj();
}
//-----------------------------------------------------------------------------
Object& add_child_obj( Array& parent )
{
    parent.push_back( Object() );

    return parent.back().get_obj();
}

//-----------------------------------------------------------------------------
Array& add_array( Object& obj, const string& name )
{
    obj.push_back( Pair( name, Array() ) );

    return obj.back().value_.get_array();
}

//-----------------------------------------------------------------------------
bool same_name( const Pair& pair, const string& name )
{
    return pair.name_ == name;
}

//-----------------------------------------------------------------------------
const Value& find_value( const Object& obj, const string& name )
{
    Object::const_iterator i = find_if( obj.begin(), obj.end(), bind( same_name, _1, ref( name ) ) );

    if( i == obj.end() ) return Value::null;

    return i->value_;
}

//-----------------------------------------------------------------------------
const Array& find_array( const Object& obj, const string& name )
{
    return find_value( obj, name ).get_array();
}


//-----------------------------------------------------------------------------
int find_int( const Object& obj, const string& name )
{
    return find_value( obj, name ).get_int();
}

//-----------------------------------------------------------------------------
string find_str( const Object& obj, const string& name )
{
    return find_value( obj, name ).get_str();
}


//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------

void read_dhkey( const Object& obj, DHKEY& dhkey )
{

    vector<TOKEN>* tokens = new vector<TOKEN>();

    //dhkey.keysel = find_str( obj, "keysel" );
    dhkey.private_key = find_str( obj, "private" );
    dhkey.public_key = find_str( obj, "public" );
    dhkey.grpsrc = find_str( obj, "grpsrc" );
    dhkey.grpdst = find_str( obj, "grpdst" );

    const Array& tokens_array( find_array(  obj, "tokens" ) );

    TOKEN record;

    for( unsigned int i = 0; i < tokens_array.size(); ++i )
    {
        read_token(tokens_array[i].get_obj(), record );
        tokens->push_back(record);
    }

    dhkey.tokens = tokens;
}

//-----------------------------------------------------------------------------
void read_token( const Object& obj, TOKEN& record )
{
    record.token = find_str( obj, "token" );
    record.label = find_str( obj, "label" );
}

//-----------------------------------------------------------------------------
void write_dhkeys( const char* file_name, vector<ResultTokens>dhkey)
{
    Object root_obj;

    Array& dhkey_array( add_array( root_obj, "values" ) );

    for (vector<ResultTokens>::iterator it = dhkey.begin(); it != dhkey.end(); ++it)
    {
        write_dhkey( dhkey_array,  *it );
    }

    ofstream os( file_name );

    write_formatted( root_obj, os );

    os.close();
}

//-----------------------------------------------------------------------------
void write_dhkey( Array& a, const ResultTokens& dhkey )
{
    Object& addr_obj( add_child_obj( a ) );

    add_pair( addr_obj, "label", dhkey.label);
    add_pair( addr_obj, "token",dhkey.token );
    add_pair( addr_obj, "grpsrc",dhkey.grpsrc);
    add_pair( addr_obj, "grpdst",dhkey.grpdst);

}


vector< TOKEN > read_tokens( Value value)
{
    Object root_obj( value.get_obj() );  // a 'value' read from a stream could be an JSON array or object

    const Array& tokens_array( find_array( root_obj, "tokens" ) );

    vector< TOKEN > tokens;

    for( unsigned int i = 0; i < tokens_array.size(); ++i )
    {
        TOKEN record;
        read_token(tokens_array[i].get_obj(), record );
        tokens.push_back( record );
    }

    return tokens;
}

//-----------------------------------------------------------------------------
vector< DHKEY > read_dhkeys( const char* file_name )
{
   ifstream is( file_name );

   Value value;

    read( is, value );

    Object root_obj( value.get_obj() );  // a 'value' read from a stream could be an JSON array or object

    const Array& dhlkey_array( find_array( root_obj, "values" ) );

    vector< DHKEY > dhkeys;

    for( unsigned int i = 0; i < dhlkey_array.size(); ++i )
    {
        DHKEY dh;
        read_dhkey(dhlkey_array[i].get_obj() , dh );

        dhkeys.push_back( dh );
    }

    return dhkeys;
}

#endif
