#ifndef HBEL_H_INCLUDED
#include <strings.h>
#include "hbel.h"
#include "dhjson.h"
#include "json_spirit.h"
#include <fstream>

//-----------------------------------------------------------------------------
void read_hbel_token( const Object& obj, HBELTOKEN& record )
{
    record.token = find_str( obj, "token" );
    record.label = find_str( obj, "label" );
}

//-----------------------------------------------------------------------------
void read_hbel_def( const Object& obj, HBELDEF& record )
{
    record.hbel_public_key = find_str( obj, "hbel_public_key" );
    record.hbel_grpsrc = find_str( obj, "hbel_grpsrc" );
    record.hbel_grpdst = find_str( obj, "hbel_grpdst" );
}

//-----------------------------------------------------------------------------
void read_hbel_node( const Object& obj, HBELNODE& record )
{
    record.hbeltokens  = new vector<HBELTOKEN>();

    record.private_key = find_str( obj, "private" );
    record.public_key = find_str( obj, "public" );
    record.grpsrc      = find_str( obj, "grpsrc" );
    record.grpdst      = find_str( obj, "grpdst" );

    // load of all tokens
    const Array& tokens_array( find_array(  obj, "tokens" ) );

    HBELTOKEN tkn;

    for( unsigned int i = 0; i < tokens_array.size(); ++i )
    {
        read_hbel_token(tokens_array[i].get_obj(), tkn);
        record.hbeltokens->push_back(tkn);
    }
}
//-----------------------------------------------------------------------------
HBEL* read_hbel_tree( const char* file_name )
{

    HBEL* nodo = new HBEL;

    ifstream is( file_name );

    Value value;

    read( is, value );

    Object root_obj( value.get_obj() );

    const Array& hbel_node_array( find_array( root_obj, "values" ) );

    vector< HBELNODE >* nodes =  new vector< HBELNODE >();

    for( unsigned int i = 0; i < hbel_node_array.size(); ++i )
    {
        HBELNODE nd;
        read_hbel_node(hbel_node_array[i].get_obj() , nd );

        nodes->push_back(nd);
    }

    const Array& hbeldefs_array( find_array( root_obj, "hbeldefs" ) );

    vector< HBELDEF >* hbeldefnodes =  new vector< HBELDEF >();

    for( unsigned int i = 0; i < hbeldefs_array.size(); ++i )
    {
        HBELDEF nd;
        read_hbel_def(hbeldefs_array[i].get_obj() , nd );
        hbeldefnodes->push_back(nd);
    }

    nodo->node = nodes;
    nodo->def = hbeldefnodes;
    return nodo;

}

//-----------------------------------------------------------------------------

#endif
