#ifndef HELPER_H_INCLUDED
#include "helper.h"

#include <vector>

#include <iostream>
#include <cassert>
#include <algorithm>
#include <fstream>
#include <boost/bind.hpp>

// Cripto++ Header
#include "aes.h"
#include "modes.h"
#include "sha.h"
#include "osrng.h"
#include "files.h"
#include "filters.h"
#include "md5.h"
#include "hex.h"
#include "hmac.h"
#include "base64.h"
#include "dh.h"
#include "secblock.h"

//-----------------------------------------------------------------------------
char* bytes(byte* token, int len)
{
    int size = (len * 2) + 1;
    char* str = new char[size];
    memset(str, 0, size);

    char temp[4];

    for(int i = 0; i< len; i++)
    {
        sprintf(temp, "%2.2x ", (int)token[i]);
        str[2*i]=temp[0];
        str[(2*i)+1]=temp[1];
    }
    return str;
}

//-----------------------------------------------------------------------------
SecByteBlock DecodeHexString(const char *hex)
{
    StringSource ss(hex, true, new HexDecoder);
    SecByteBlock result((size_t)ss.MaxRetrievable());
    ss.Get(result, result.size());
    return result;
}

//-----------------------------------------------------------------------------
string EncodeHexString(SecByteBlock source)
{
    CryptoPP::HexEncoder encoder;
    std::string output;
    encoder.Attach( new CryptoPP::StringSink( output ) );
    encoder.Put( source, LEN );
    encoder.MessageEnd();
    return output;
}

//-----------------------------------------------------------------------------
CryptoPP::RandomPool & GlobalRNG(void)
{
    static CryptoPP::RandomPool randomPool;
    return randomPool;
}

//-----------------------------------------------------------------------------
string decodeAESKey(SecByteBlock source, byte* key)
{
    string encrypted = EncodeHexString(source);
    string output;
     //parametri aes
    ECB_Mode<AES>::Decryption decryptor(key, 16);
    //effettuo il decode dell chiave in hex per il decrypt
    StringSource(encrypted, true, new HexDecoder(new StringSink(output)));

    return output;
}


//-----------------------------------------------------------------------------
string decodeAESKey(string encrypted, byte* key)
{
    string output;
     //parametri aes
    ECB_Mode<AES>::Decryption decryptor(key, 16);
    //decode of the key in hex in order to prepare for the decryption
    string hexText;
    StringSource(encrypted, true, new HexDecoder(new StringSink(hexText)));
    StringSource ss2(hexText, true, new StreamTransformationFilter(decryptor, new HexEncoder(new StringSink(output), false)));
    return output;
}

//-----------------------------------------------------------------------------
string encodeAESKey(SecByteBlock source, byte* key)
{
    string decPrivateKey =  EncodeHexString(source);
    string hexDecPrivateKey;
    StringSource(decPrivateKey, true, new HexDecoder(new StringSink(hexDecPrivateKey)));

    string encrypted;
    string decrypted;
    string hexPrivateKey;

    byte* key2;
    key2 = key;

    // Opzioni per l'AES ECB mode
    ECB_Mode<AES>::Encryption encryptor(key2, 16);
    ECB_Mode<AES>::Decryption decryptor(key2, 16);

    StringSource ss(hexDecPrivateKey, true, new StreamTransformationFilter(encryptor,new HexEncoder(new StringSink(encrypted),false)));
    return encrypted;
}


//-----------------------------------------------------------------------------
byte* KeyAgreement(char* DHprivateKey, char* DHpublicKey, char* OWNprivateKey)
{

    Integer iModulus(DH_MODULUS_DEC);
    DH d(iModulus, 2);

    SecByteBlock key;        //user private key
    string decPrivateKey;    //dh private key decrypted
    key = DecodeHexString(OWNprivateKey); //user private key in hex

    ECB_Mode<AES>::Decryption decryptor(key, 16);

    decPrivateKey = decodeAESKey(DHprivateKey, key);

    //key agreement
    SecByteBlock secretKey(d.AgreedValueLength());
    const char* strPrivateKey = decPrivateKey.c_str();

    SecByteBlock privateK = DecodeHexString(strPrivateKey);
    SecByteBlock publicK = DecodeHexString(DHpublicKey);

    d.Agree(secretKey, privateK, publicK);

    //create hash with 16byte
    byte* secretKey16 = new byte[16];
    for (int i = 0; i < 16; i++) {
        secretKey16[i] = (byte) (secretKey[i * 2] ^ secretKey[(i * 2) + 1]);
    }

    return secretKey16;

}


//-----------------------------------------------------------------------------
byte* PlainKeyAgreement(char* DHprivateKey, char* DHpublicKey, char* OWNprivateKey)
{

    Integer iModulus(DH_MODULUS_DEC);
    DH d(iModulus, 2);


    SecByteBlock key;
    string decPrivateKey;
    string encPrivateKey;
    key = DecodeHexString(OWNprivateKey);

    SecByteBlock privateK = DecodeHexString(DHprivateKey);
    SecByteBlock publicK = DecodeHexString(DHpublicKey);

    SecByteBlock secretKey(d.AgreedValueLength());
    d.Agree(secretKey, privateK, publicK);

    byte* secretKey16 = new byte[16];
    for (int i = 0; i < 16; i++) {
        secretKey16[i] = (byte) (secretKey[i * 2] ^ secretKey[(i * 2) + 1]);
    }
    return secretKey16;

}
//-----------------------------------------------------------------------------
char* generateRandomLabel(int len, int start , int range)
{
    char* label = new char[len + 1];

    for(int i = 0; i< len; i++)
    {
         label[i] = start + rand() % range;
    }
    label[len] = 0;
    return label;

}

//-----------------------------------------------------------------------------
char* generateRandomKey(int len, int start, int range)
{
    const char *values = "0123456789ABCDEF";

    char* key = new char[len + 1];

    for(int i = 0; i< len; i++)
    {
         key[i] = values[start + rand() % range];
    }
    key[len] = 0;



    return key;
}


void tokenize(const string& str, vector<string>& tokens, const string& delimiters)
{
    // Skip delimiters at beginning.
    string::size_type lastPos = str.find_first_not_of(delimiters, 0);
    // Find first "non-delimiter".
    string::size_type pos     = str.find_first_of(delimiters, lastPos);
    while (string::npos != pos || string::npos != lastPos)
    {
        // Found a token, add it to the vector.
        tokens.push_back(str.substr(lastPos, pos - lastPos));
        // Skip delimiters.  Note the "not_of"
        lastPos = str.find_first_not_of(delimiters, pos);
        // Find next "non-delimiter"
        pos = str.find_first_of(delimiters, lastPos);
    }
}


//-----------------------------------------------------------------------------
byte* createToken(byte* keySource, char* keyDestination, char* label) {

    byte* token = new byte[16];
    byte* keyS = keySource;
    byte* keyD = hexChartoHexBytes(keyDestination);

    byte hashOutputBuffer[ CryptoPP::SHA::DIGESTSIZE ];
    CryptoPP::HMAC<CryptoPP::SHA1> hmac(keyS, 16);
    hmac.Update((byte*)label, strlen(label));
    hmac.Final(hashOutputBuffer);

    //create hash
    for (int i = 0; i < 16; i++)
        token[i] = (byte) (keyD[i] ^ hashOutputBuffer[i]);

    return token;
}


//-----------------------------------------------------------------------------
byte* createTokenHBEL(byte* keySource, char* keyDestination, char* label) {

    byte* token = new byte[16];
    byte* keyS = keySource;
    byte* keyD = hexChartoHexBytes(keyDestination);

    byte hashOutputBuffer[ CryptoPP::SHA::DIGESTSIZE ];
    CryptoPP::HMAC<CryptoPP::SHA1> hmac(keyS, 16);
    hmac.Update((byte*)label, strlen(label));
    hmac.Final(hashOutputBuffer);

    //create hash
    for (int i = 0; i < 16; i++)
        token[i] = (byte) (keyD[i] ^ hashOutputBuffer[i]);

    return token;
}

//-----------------------------------------------------------------------------
byte* rebuildKey(byte* keySource, byte* token, char* label)
{

    byte* destKey = new byte[16];
    byte* keyS = keySource;


    byte hashOutputBuffer[ CryptoPP::SHA::DIGESTSIZE ];
    CryptoPP::HMAC<CryptoPP::SHA1> hmac(keyS, 16);
    hmac.Update((byte*)label, strlen(label));
    hmac.Final(hashOutputBuffer);

    //create hash
    for (int i = 0; i < 16; i++)
        destKey[i] = (byte) (token[i] ^ hashOutputBuffer[i]);

    return destKey;
}
//-----------------------------------------------------------------------------
byte* hash(string key)
{
    CryptoPP::MD5 hash;
    byte* digest = new byte [CryptoPP::MD5::DIGESTSIZE ];

    hash.CalculateDigest( digest, (byte*)key.c_str(), key.length() );
    return digest;
}
//-----------------------------------------------------------------------------
byte* hash(byte* key)
{
    CryptoPP::MD5 hash;
    byte* digest = new byte [CryptoPP::MD5::DIGESTSIZE ];

    hash.CalculateDigest( digest, key, sizeof(key) );
    return digest;
}
//-----------------------------------------------------------------------------
byte* generateOwnerKey(string pin)
{
   return hash((string)"estevan&flavio" + pin);
}
//-----------------------------------------------------------------------------
string generateDiffieHellmanKeysPair(string pin)
{

    byte* hashPin = hash(bytes(hash(pin)));
    byte* key = new byte[16];

    Integer iModulus(DH_MODULUS_DEC);
    Integer iGenerator(DH_BASE);
    DH dh(iModulus, iGenerator);

    SecByteBlock privKey(dh.PrivateKeyLength());
    SecByteBlock publKey(dh.PublicKeyLength());

    dh.GenerateKeyPair(GlobalRNG(), privKey, publKey);

    key = generateOwnerKey(pin);

    string encpk = encodeAESKey(privKey,key);

    string s = "{  \"public\": \"" + EncodeHexString(publKey) + (string)"\"," +
                "\"private\": \"" + encpk                    + (string)"\"," +
                "\"hash\": \""    + bytes(hashPin,16) + (string)"\"}";

    return s;


}

int decryptSEL(char* password,  char*inputFileName,
                 char* outputFileName)
{
    SecByteBlock key = DecodeHexString(password);


    if(password != NULL)
    {
        decryptFile(key, inputFileName, outputFileName);
        return 0;
    }
    return -1;
}


byte* prepareEncryption(char* filein, char* fileout, char* pin)
{
    /**
        Numero di utenti con cui condividere
    */
    int nUser = 0;

    /**
        Result dei key agreement
    */
    byte* result = NULL;

    /**
        Result dei token
    */
    byte* token = NULL;

    /**
        temp key for token creation
    */
    byte* aeskey = NULL;

    /**
        Chiper key for encryption
    */
    byte* resultKey = NULL;


    vector< ResultTokens > output;


    ifstream in;

    srand(time(0));
    in.open (filein, ios::in | ios::binary);

    if(in.fail())
    {
        cerr << "ERROR reading file " << filein << endl;
        return NULL;
    }

    vector<DHKEY>* new_dhkeys = new vector<DHKEY>(read_dhkeys( filein ));

    nUser = new_dhkeys->size();


    if(nUser == 0)
    {

        resultKey = generateOwnerKey(pin);
    }

    else
    {

        char* randomKey = generateRandomKey();
        char* newlabel;
        resultKey = hexChartoHexBytes(randomKey);

        for (vector<DHKEY>::iterator it = new_dhkeys->begin(); it != new_dhkeys->end(); ++it)
        {
           result = KeyAgreement((char*)it->private_key.c_str(), (char*)it->public_key.c_str(), bytes(generateOwnerKey(pin)));

           newlabel = generateRandomLabel();

           vector< TOKEN >*vt = it->tokens;
           if(vt->size() == 0)
            {
                /*
                Share a file with a single user
                */
                if (it->grpsrc == "") {
                    resultKey = result;
                }
                /*
                    Share file with more than one users, tokens are almost stored on db.
                */
                else
                {
                    token =  createToken(result, randomKey,  newlabel);
                    //write token
                    ResultTokens record;

                    record.label = newlabel;
                    record.token = bytes(token);
                    record.grpsrc  = it->grpsrc;
                    record.grpdst  = it->grpdst;
                    output.push_back(record);
                }
            }

            // there are more tokes (>=1)
            else
            {
                aeskey = result;


                for (vector< TOKEN >::iterator t = vt->begin(); t != vt->end(); ++t)
                {
                    aeskey =  rebuildKey(aeskey, hexChartoHexBytes(t->token.c_str()),  (char*)t->label.c_str());
                }

                if (it->grpsrc == "") {
                    resultKey = aeskey;
                }
                else {
                    token =  createToken(aeskey, randomKey,  newlabel);

                    //write token
                    ResultTokens record;

                    record.label = newlabel;
                    record.token = bytes(token);
                    record.grpsrc  = it->grpsrc;
                    record.grpdst  = it->grpdst;

                    output.push_back(record);
                }
            }

        }
    }
    // ... writing token over JSON formated file
    write_dhkeys(fileout, output);
	return hash(resultKey);
}


byte* prepareDecryption(char* jsonin, char* pin)
{
    /**
        Numero di utenti con cui condividere io + nUser
    */
    int nUser = 0;

    /**
        Result dei key agreement
    */
    byte* result = NULL;

    /**
        Result dei token
    */
    byte* token = NULL;

    /**
        temp key for token creation
    */
    byte* aeskey = NULL;

    /**
        Chiper key for encryption
    */
    byte* resultKey = NULL;


    vector< ResultTokens > output;


    ifstream in;

    srand(time(0));//seleziona il seme
    in.open (jsonin, ios::in | ios::binary);

    if(in.fail())
    {
        cerr << "ERRROR reading file " << jsonin << endl;
        return NULL;
    }

    vector< DHKEY > new_dhkeys( read_dhkeys( jsonin ) );

    nUser = new_dhkeys.size();

    // Upload of a file w/o sharing
    // no shared files, only me.....
    if(nUser == 0)
    {
       // return generateOwnerKey(pin);
        return hash(generateOwnerKey(pin));
    }


    else //if(nUser > 1)
    {
        vector< TOKEN >*vt = new_dhkeys[0].tokens;

        // If there are no tokens in JSON then
        resultKey = KeyAgreement(
                                (char*)new_dhkeys[0].private_key.c_str(),
                                (char*)new_dhkeys[0].public_key.c_str(),
                                bytes( generateOwnerKey(pin))
                            );
        aeskey = resultKey;

        string dstgrp = new_dhkeys[0].grpdst;

        vector<string> td;
        tokenize(dstgrp,td, "-");

        if(td.size() == 2)
        {
            aeskey = hash(aeskey);
        }


        if(vt->size() > 0)
        {
            for (vector< TOKEN >::iterator t = vt->begin(); t != vt->end(); ++t)
            {
                aeskey =  rebuildKey(aeskey, hexChartoHexBytes(t->token.c_str()),  (char*)t->label.c_str());
            }

        }
    }
    return hash(aeskey);
}


byte* prepareDecryptionHBEL(char* jsonin, char* pin)
{
    /**
        Numero di utenti con cui condividere io + nUser
    */
    int nUser = 0;

    /**
        Result dei key agreement
    */
    byte* result = NULL;

    /**
        Result dei token
    */
    byte* token = NULL;

    /**
        temp key for token creation
    */
    byte* aeskey = NULL;

    /**
        Chiper key for encryption
    */
    byte* resultKey = NULL;


    vector< ResultTokens > output;


    ifstream in;

    srand(time(0));//seleziona il seme
    in.open (jsonin, ios::in | ios::binary);

    if(in.fail())
    {
        cerr << "ERRROR reading file " << jsonin << endl;
        return NULL;
    }
    vector< DHKEY > new_dhkeys( read_dhkeys( jsonin ) );

    nUser = new_dhkeys.size();

    if(nUser == 0)
    {
        return hash(generateOwnerKey(pin));
    }


    else
    {
        vector< TOKEN >*vt = new_dhkeys[0].tokens;
        resultKey = KeyAgreement(
                                (char*)new_dhkeys[0].private_key.c_str(),
                                (char*)new_dhkeys[0].public_key.c_str(),
                                bytes( generateOwnerKey(pin))
                            );
        aeskey = resultKey;
        string dstgrp = new_dhkeys[0].grpdst;

        vector<string> td;
        tokenize(dstgrp,td, "-");

        if(td.size() == 2)
        {
            aeskey = hash(aeskey);
        }


        if(vt->size() > 0)
        {
            for (vector< TOKEN >::iterator t = vt->begin(); t != vt->end(); ++t)
            {
                aeskey =  rebuildKey(aeskey, hexChartoHexBytes(t->token.c_str()),  (char*)t->label.c_str());
            }

        }
    }

    return aeskey;
}


//-----------------------------------------------------------------------------
void encryptFile(byte* password, const char* inputFileName,
                 const char* outputFileName)
{
    byte iv[LEN];             // real IV used - set to zero

    memset(iv, 0, LEN);
    try {

        // create object for encrypting
        AES::Encryption aesEncryption(password, CryptoPP::AES::DEFAULT_KEYLENGTH);

        CBC_Mode_ExternalCipher::Encryption cbcEncryption(aesEncryption, iv);

        StreamTransformationFilter *encryptor;
        encryptor = new StreamTransformationFilter(cbcEncryption,  new FileSink(outputFileName) );

        // "bind" a file and encrypt one
        FileSource(inputFileName, true, encryptor);

    } catch (CryptoPP::Exception &e) {
        cerr << e.what() << endl;
        return;
    }
}
//-----------------------------------------------------------------------------
void decryptFile(byte* password, const char*inputFileName,
                 const char* outputFileName)
{
   byte iv[LEN];
   memset(iv, 0, LEN); // very correct - in fact the encryptor prefixes file

   try {
        CryptoPP::AES::Decryption aesDecryption( password, CryptoPP::AES::DEFAULT_KEYLENGTH);
        CryptoPP::CBC_Mode_ExternalCipher::Decryption cbcDecryption(aesDecryption, iv);

        StreamTransformationFilter *decryptor =  new StreamTransformationFilter(cbcDecryption, new FileSink(outputFileName));

        FileSource(inputFileName, true, decryptor);

    } catch (CryptoPP::Exception &e) {
        cerr << e.what() << endl;
        return;
    }
}


//-----------------------------------------------------------------------------
byte* hexChartoHexBytes(const char* str )
{
    int len = strlen(str)/2;
    byte* buffer = new byte[len];

    for (int i = 0; i< len; i++) {
        buffer[i] = 0xFF;

        switch (toupper(str[i*2])) {
        case '0':
            buffer[i] = buffer[i] & 0x0F;
            break;
        case '1':
            buffer[i] = buffer[i] & 0x1F;
            break;
        case '2':
            buffer[i] = buffer[i] & 0x2F;
            break;
        case '3':
            buffer[i] = buffer[i] & 0x3F;
            break;
        case '4':
            buffer[i] = buffer[i] & 0x4F;
            break;
        case '5':
            buffer[i] = buffer[i] & 0x5F;
            break;
        case '6':
            buffer[i] = buffer[i] & 0x6F;
            break;
        case '7':
            buffer[i] = buffer[i] & 0x7F;
            break;
        case '8':
            buffer[i] = buffer[i] & 0x8F;
            break;
        case '9':
            buffer[i] = buffer[i] & 0x9F;
            break;
        case 'A':
            buffer[i] = buffer[i] & 0xaF;
            break;
        case 'B':
            buffer[i] = buffer[i] & 0xbF;
            break;
        case 'C':
            buffer[i] = buffer[i] & 0xcF;
            break;
        case 'D':
            buffer[i] = buffer[i] & 0xdF;
            break;
        case 'E':
            buffer[i] = buffer[i] & 0xeF;
            break;
        case 'F':
            buffer[i] = buffer[i] & 0xFF;
            break;
        }


        switch (toupper(str[(i*2)+1])) {
        case '0':
            buffer[i] = buffer[i] & 0xF0;
            break;
        case '1':
            buffer[i] = buffer[i] & 0xF1;
            break;
        case '2':
            buffer[i] = buffer[i] & 0xF2;
            break;
        case '3':
            buffer[i] = buffer[i] & 0xF3;
            break;
        case '4':
            buffer[i] = buffer[i] & 0xF4;
            break;
        case '5':
            buffer[i] = buffer[i] & 0xF5;
            break;
        case '6':
            buffer[i] = buffer[i] & 0xF6;
            break;
        case '7':
            buffer[i] = buffer[i] & 0xF7;
            break;
        case '8':
            buffer[i] = buffer[i] & 0xF8;
            break;
        case '9':
            buffer[i] = buffer[i] & 0xF9;
            break;
        case 'A':
            buffer[i] = buffer[i] & 0xFa;
            break;
        case 'B':
            buffer[i] = buffer[i] & 0xFb;
            break;
        case 'C':
            buffer[i] = buffer[i] & 0xFc;
            break;
        case 'D':
            buffer[i] = buffer[i] & 0xFd;
            break;
        case 'E':
            buffer[i] = buffer[i] & 0xFe;
            break;
        case 'F':
            buffer[i] = buffer[i] & 0xFF;
            break;
        }


    }

    return buffer;
}

#endif
