package clientFunction;

import java.sql.ResultSet;
import java.sql.Statement;
import java.util.LinkedList;
import java.util.StringTokenizer;

/**
 *  This class allows to buils the access graph for a specified user that wishes to outsource his resources 
 */
public class AccessGraph {

	
	private LinkedList<ACL> ACLList;
	/** Type: 0->BEL 1->SEL */
	int type;
	private LinkedList<ACL> albero;

	public AccessGraph(int type){
		ACLList = new	LinkedList<ACL>();
		this.type=type;
		buildAccessGraph();
	}
	
	public void addAcl(ACL newElement) {
		LinkedList<String> ll=new LinkedList<String>();
		for(int i=0;i<ACLList.size();++i)
			ll.add(ACLList.get(i).getID());
		if(!ll.contains(newElement.getID()))
			ACLList.add(newElement);
	}
	
	public void addAcl(String id, LinkedList<String> userList) {
		ACLList.add(new ACL(id,userList));
	}
	
	public void addAcl(String id) {
		ACLList.add(new ACL(id));
	}

	public LinkedList<ACL> getAllAcl() {
		return ACLList;
	}
	
	public LinkedList<ACL> getAlbero() {
		return albero;
	}

	public LinkedList<ACL> getAclListBySize(int size){
		LinkedList<ACL> acl = new LinkedList<ACL>();
		if(size>0){
			for(int i=0; i < ACLList.size(); i++){
				if(ACLList.get(i).getSize()==size)
					acl.add(ACLList.get(i));
			}
		}	
		if(acl.size()==0)
			return null;	
		return acl;
	}	
	

	private boolean exist(String ID){
		for(int i = 0;i < ACLList.size(); i++){
			if(ACLList.get(i).getID().equals(ID))
				return true;
		}
		return false;
	}
	
	private ACL getAclById(String ID){
		for(int i = 0;i < ACLList.size(); i++){
			if(ACLList.get(i).getID().equals(ID))
				return ACLList.get(i);
		}
		return null;
	}
	
	public int getNmax()
	{
		int max=0;
		for (int i=0;i<ACLList.size();++i)
			if(ACLList.get(i).getSize()>max)
				max=ACLList.get(i).getSize();
		
		return max;
	}
	
	public void buildAccessGraph() {		
		DBConnection connection = new DBConnection();
		try {
			Statement stmt = connection.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rset;	
			
			if(type==0) // BEL
				rset = stmt.executeQuery("SELECT groupidsource, groupiddestination from tokentable where level='BEL'");
			else        // SEL
				rset = stmt.executeQuery("SELECT groupidsource, groupiddestination from tokentable where level='SEL'");	
			
			
			ResultSet temp = rset;
		int cont=0;
			while (temp.next()){
			            ACL acl1= new ACL(temp.getString(1));
			            acl1.addUsers(getUserFromIdGroup(temp.getString(1)));
			            ACL acl2= new ACL(temp.getString(2));
			            acl2.addUsers(getUserFromIdGroup(temp.getString(2)));
			            ++cont;
			            
			            if (!exist(acl1.getID()))
			            {     
			            	 acl1.addDescendent(acl2);
			            	 addAcl(acl1);
			    		}
			            
			            else
			            {
			            	for(int i = 0;i < ACLList.size(); i++){
			        			if(ACLList.get(i).getID().equals(acl1.getID()))
			        				{ACLList.get(i).addDescendent(acl2);
			        				 
			        				}
			            }}	
			            
			            if (!exist(acl2.getID()))
			            {     
			            	acl2.addAncestor(acl1);
			            	 addAcl(acl2);
			    		}	
			            
			            else
			            {
			            	for(int i = 0;i < ACLList.size(); i++){
			        			if(ACLList.get(i).getID().equals(acl2.getID()))
			        				{ACLList.get(i).addAncestor(acl1);
			        				}
			            }}
			}
			rset.close();
			stmt.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		connection.closeConnection();
	}
	
	private void buildPath(ACL src, ResultSet rset) {
		
		ACL des = null;
		int pos;
		try {
			if (rset != null) {
				do {
					if (rset.getString(1).equals(src.getID())) {
						if (!exist(rset.getString(2))) {
							des = new ACL(rset.getString(2), src.getUserList());
							addAcl(des);
						} else {
							des = getAclById(rset.getString(2));
							des.addUsers(src.getUserList());
						}
						src.addDescendent(des);		
						des.addAncestor(src);
						
						pos=rset.getRow();
						rset.first();
						buildPath(des, rset);
						rset.absolute(pos);
					}
				}while (rset.next());
			}
		} catch (Exception e) {
			System.err.println(e.toString());
		}

	}
	
	private static LinkedList<String> getUserFromIdGroup(String ID) {
		StringTokenizer group=new StringTokenizer(ID,"-");
		LinkedList<String> list=new LinkedList<String>();
        while (group.hasMoreTokens())
        {
        	list.add(group.nextToken());
        }
        
        return list;
	}

  public ACL getACLfromID(String key)
   {
	  for (int i=0;i<ACLList.size();++i)
		  if (key.equals(ACLList.get(i).getID()))
			  return ACLList.get(i);
	  return null;
	  }
  
  public ACL getACLfromNum(int n)
  {
	  for (int i=0;i<albero.size();++i)
		  if (n==(albero.get(i).getNum()))
			  return albero.get(i);
	  return null;
	  }
  
  public int numera(String id, int n,LinkedList<ACL> albero)
  {		
	  ACL acl=new ACL(); 
	  for(int i=0;i<albero.size();++i)
		  if(albero.get(i).getID().equals(id))
			  acl=albero.get(i);
			  
	  if(acl.getDescendents().size()>0)
	  { 
		     for(int i=0;i<acl.getDescendents().size();++i)
			 {   
				     n=numera(acl.getDescendents().get(i).getID(),n,albero);
			 }
	  }
	  acl.setNum(n);
	  n++;
	  return n;
  }
  
  public LinkedList<ACL> TrasformaInAlbero()
  {
	  
	  LinkedList<ACL> albero=new LinkedList<ACL>();
	  ACL radice=new ACL("radice");
	  for(int i=0;i<this.getAclListBySize(2).size();++i)
		  radice.addDescendent(this.getAclListBySize(2).get(i));
	  LinkedList<String> giafatto=new LinkedList<String>();
	  CreaAlbero(albero,giafatto,radice);
	  
	    
	
	  for(int j=0;j<albero.size();++j)
		  if(albero.get(j).getID().equals("radice"))
			    albero.remove(j);
			  
	  ACL rad=new ACL("radice");
	  for(int i=0;i<this.getAclListBySize(2).size();++i)
		  rad.addDescendent(this.getAclListBySize(2).get(i));
	  albero.add(rad);
	
	  this.numera("radice", 1, albero);
	 
	  for(int i=0;i<albero.size();++i)
		  for(int j=0;j<albero.size()-1;++j)
		  {
			  ACL supp=new ACL();
			  if(albero.get(j).getNum()>albero.get(j+1).getNum())
			  {
				  supp=albero.get(j);
				  albero.set(j, albero.get(j+1));
				  albero.set(j+1, supp);
			  }
		  }
	  
	  this.albero=albero;
	  return albero;
  }
  
  
  private boolean ACLcontenutoInAlbero(LinkedList<ACL> list,ACL acl)
  {
	  for(int i=0;i<list.size();++i)
		  if(list.get(i).getID().equals(acl.getID()))
			  return true;
	  return false; 
  }
  
  private ACL getACLdaAlbero(LinkedList<ACL> albero,String id)
  {
	  for(int i=0;i<albero.size();++i)
		  if(albero.get(i).getID().equals(id))
			  return albero.get(i);
	  return null;
  }
  
  public void CreaAlbero(LinkedList<ACL> albero,LinkedList<String> giafatto,ACL acl)
  {
	 if(!acl.getID().equals("radice"))
	    acl=this.getACLfromID(acl.getID());
	 if(acl.getDescendents().size()==0)
	 {	
		 if(!albero.contains(acl))
		  albero.add(new ACL(acl.getID()));}
	 else	 
	 {   ACL nuovo=null;
		 if(!this.ACLcontenutoInAlbero(albero, acl)){
		     nuovo=new ACL(acl.getID());
		     
		 }
		 else
			 {nuovo=this.getACLdaAlbero(albero, acl.getID());
			 
			 }
		 for(int i=0;i<acl.getDescendents().size();++i)
	 {   
		 
		  
		
		 boolean t=false;
		 ACL figlio=acl.getDescendents().get(i);
		
		 
		 if(!giafatto.contains(figlio.getID()))
			 {   
			     nuovo.addDescendent(new ACL(figlio.getID()));
			     
				 giafatto.add(figlio.getID());
				 t=true;
			 }
		 if(i==acl.getDescendents().size()-1)
		 {
			 albero.add(nuovo);
			
		 }	 
			if(t==true)
			{
				 CreaAlbero(albero,giafatto,acl.getDescendents().get(i));
				 t=true;
				 
			}
		 }}
	  return;
}
  
public void CreaIntervalli()
{
	this.TrasformaInAlbero();
	
	for(int i=1;i<albero.size()+1;++i)
	   this.getACLfromNum(i).setIntervallo(this.getACLfromNum(i).getEstremo(this.getAlbero()), i);
	    
	for(int i=0;i<albero.size();++i)
		if(!albero.get(i).getID().equals("radice"))
			{this.getACLfromID(albero.get(i).getID()).setIntervallo(albero.get(i).getIntervalli().get(0).getMin(), albero.get(i).getIntervalli().get(0).getMax());
			 this.getACLfromID(albero.get(i).getID()).setNum(albero.get(i).getNum());
			}

	for(int i=this.getNmax();i>2;--i)
	{   
		LinkedList<ACL> list=this.getAclListBySize(i);
		if(list!=null)
		for(int j=0;j<list.size();++j)
			for(int t=0;t<list.get(j).getAncestor().size();++t)
				this.getACLfromID(list.get(j).getAncestor().get(t).getID()).setIntervalli(list.get(j).getIntervalli());
			
	}
	SalvaIntervalliInDB();
}

	public void UpdateToken(DBConnection connection,String s,String tokenID)
	{
		try {
			Statement stmt = connection.getConnection().createStatement();

			stmt.execute("Update tokentable set interval='"+s+ "' where tokenid = '"+tokenID+"'");
			
			stmt.close();

		} catch (Exception e) {
			System.err.println(e.toString());
			
		}
		
	}

	public void SalvaIntervalliInDB()
	{
		DBConnection connection = new DBConnection();
		try {
			Statement stmt = connection.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rset;	
			if(this.type==0)
				rset = stmt.executeQuery("SELECT groupidsource,groupiddestination,tokenid from tokentable where level='BEL'");
			else
				rset = stmt.executeQuery("SELECT groupidsource,groupiddestination,tokenid from tokentable where level='SEL'");				
			ResultSet temp = rset;
		
			while (temp.next()){
				    ACL aclsource=this.getACLfromID(temp.getString(1));
				    ACL acldest=this.getACLfromID(temp.getString(2));
				    if(aclsource.getSize()==2)
			    	   UpdateToken(connection,acldest.CreaStringaIntervalli()+"*"+aclsource.CreaStringaIntervalli(),temp.getString(3)); 
				    else
			           UpdateToken(connection,acldest.CreaStringaIntervalli(),temp.getString(3));
			
			}
			
			
			
			rset.close();
			stmt.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		
		connection.closeConnection();
		return;
	}
	
	public void CaricaIntervalli()
	{
		DBConnection connection = new DBConnection();
		try {
			Statement stmt = connection.getConnection().createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
			ResultSet rset;	
			if(this.type==0)
			rset = stmt.executeQuery("SELECT groupidsource,groupiddestination,interval from tokentable where level='BEL'");
			else
			rset = stmt.executeQuery("SELECT groupidsource,groupiddestination,interval from tokentable where level='SEL'");
	
			ResultSet temp = rset;
		
			while (temp.next()){
				    ACL aclsource=this.getACLfromID(temp.getString(1));
				    ACL acldest=this.getACLfromID(temp.getString(2));
				    String inter=temp.getString(3);
				    
				    StringTokenizer group=new StringTokenizer(inter,"*");
					LinkedList<String> list=new LinkedList<String>();
			        while (group.hasMoreTokens())
			        {
			        	list.add(group.nextToken());
			        }
			        
			        for(int i=0;i<list.size();++i)
			        {
			        	StringTokenizer group2=new StringTokenizer(list.get(i),"/"); 
			        	LinkedList<String> intervalli=new LinkedList<String>();
				        while (group2.hasMoreTokens())
				        {
				        	intervalli.add(group2.nextToken());
				        }
				        if(i==0)
				        {
				        	LinkedList<Intervallo> interv=RicavaIntervalli(intervalli);
				        	acldest.copiaIntervalli(interv);
				        	acldest.setNum(acldest.RicavaMax());
				        }
				        if(i==1)
				        {
				        	LinkedList<Intervallo> interv=RicavaIntervalli(intervalli);
				        	aclsource.copiaIntervalli(interv);
				        	aclsource.setNum(aclsource.RicavaMax());
				        }
			        }
			
			}
			
			
			
			rset.close();
			stmt.close();
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		
		connection.closeConnection();
		return;
	}
	
	
	public LinkedList<Intervallo> RicavaIntervalli(LinkedList<String> i)
	{
		LinkedList<Intervallo> inter=new LinkedList<Intervallo>();
		
		for(int j=0;j<i.size();++j)
		{
		StringTokenizer group2=new StringTokenizer(i.get(j),","); 
		LinkedList<String> valori=new LinkedList<String>();
        while (group2.hasMoreTokens())
        {
        	valori.add(group2.nextToken());
        }
        Intervallo supp=new Intervallo(Integer.parseInt(valori.get(0)),Integer.parseInt(valori.get(1)));
        inter.add(supp);
		}
		
		return inter;
	}

}
