package clientFunction;
import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.util.Arrays;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.Mac;
import javax.crypto.SecretKey;
import javax.crypto.spec.SecretKeySpec;

/**
  * This class provides static methods to encrypt/decrypt a file 
 */
public class CopyOfCryptoFunction {
	public static int BLOCK_SIZE = 4095;
	public static int AES_BLOCK_SIZE = ((BLOCK_SIZE / 16) + 1) * 16;

	public static String CRIPTOGRAPHIC_ALGORITHM = "AES";
	public static boolean encryptFile(byte[] secretKey, String filePath, String saveDirectory,boolean hash) {
		
		try {
			byte[] skey;
			byte[] encrypted = null;
			byte[] finalBlock;
			byte[] original;

			FileInputStream fin;
			BufferedInputStream dis;
			FileOutputStream fout;
			BufferedOutputStream dos;

			if(hash)
			{
				skey=hashSecretKey(secretKey);
			}
			else 
			{
				skey=secretKey;
			}
			
			SecretKeySpec skeySpec = new SecretKeySpec(skey, CRIPTOGRAPHIC_ALGORITHM);
			Cipher cipher = Cipher.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			cipher.init(Cipher.ENCRYPT_MODE, skeySpec);

			fin = new FileInputStream(filePath);
			dis = new BufferedInputStream(fin);
			fout = new FileOutputStream(saveDirectory + getFileName(filePath));
			dos = new BufferedOutputStream(fout);

			int numByte;

			original = new byte[BLOCK_SIZE];

			while (dis.available() != 0) {
				numByte = dis.read(original);

				if (numByte < AES_BLOCK_SIZE) {
					finalBlock = new byte[numByte];
					//finalBlock = Arrays.copyOf(original, numByte);
					encrypted = cipher.doFinal(finalBlock);
				} else
					encrypted = cipher.doFinal(original);

				dos.write(encrypted);
			}
			dos.flush();
			fout.close();
			fin.close();
		}
		// Catches any error conditions
		catch (Exception e) {
			System.err.println(e.toString());
			return false;
		}
		return true;
	}
	

	public static void decryptFile(byte[] secretKey, String filePath, String saveDirectory,boolean hash) {
		
		try {
			byte[] skey;
			byte[] encrypted = null;
			byte[] finalBlock;
			byte[] original;

			int numByte;

			FileInputStream fin;
			BufferedInputStream dis;
			FileOutputStream fout;
			BufferedOutputStream dos;

			fin = new FileInputStream(filePath);
			dis = new BufferedInputStream(fin);
			fout = new FileOutputStream(saveDirectory);
			dos = new BufferedOutputStream(fout);
			
			if(hash)
			{
				skey=hashSecretKey(secretKey);
			}
			else 
			{
				skey=secretKey;
			}

			SecretKeySpec skeySpec = new SecretKeySpec(skey, CRIPTOGRAPHIC_ALGORITHM);
			Cipher cipher = Cipher.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			cipher.init(Cipher.DECRYPT_MODE, skeySpec);

			encrypted = new byte[AES_BLOCK_SIZE];

			while (dis.available() != 0) {

				numByte = dis.read(encrypted);

				if (numByte < AES_BLOCK_SIZE) {
					finalBlock = new byte[numByte];
					//finalBlock = Arrays.copyOf(encrypted, numByte);
					original = cipher.doFinal(finalBlock);
				} else
					original = cipher.doFinal(encrypted);

				dos.write(original);
			}
			dos.flush();
			fout.close();
			fin.close();

		}
		// Catches any error conditions
		catch (Exception e) {
			System.err.println(e.toString());
		}
	}
	
	public static byte[] decrypt(byte[] encrypted, byte[] AESKey) {
		
		byte[] decrypted = null;
		try {
			SecretKeySpec skeySpec = new SecretKeySpec(AESKey, CRIPTOGRAPHIC_ALGORITHM);
			Cipher cipher = Cipher.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			cipher.init(Cipher.DECRYPT_MODE, skeySpec);
			decrypted = cipher.doFinal(encrypted);
		} catch (Exception e) {
			System.err.println(e.toString());
		}
		return decrypted;
	}
	
	public static byte[] createKey() {
		byte[] keyAES = null;
		try {

			KeyGenerator kgen = KeyGenerator.getInstance(CRIPTOGRAPHIC_ALGORITHM);
			kgen.init(128);
			SecretKey skey = kgen.generateKey();
			keyAES = skey.getEncoded();

		} catch (Exception e) {

			System.err.println(e.toString());

		}
		return keyAES;

	}
	
	private static String getFileName(String path) {
		StringBuffer fileName = new StringBuffer();

		for (int c = (path.length() - 1); (c != 0 && path.charAt(c) != File.separatorChar); c--)
			fileName.append(path.charAt(c));

		fileName.reverse();

		return fileName.toString();
	}
	
	public static byte[] hashSecretKey(byte[] secretKey)
	{
		byte[] hashedKey = new byte[16];
		try{
			SecretKeySpec skeySpec = new SecretKeySpec(secretKey, CRIPTOGRAPHIC_ALGORITHM);
			Mac mac = Mac.getInstance("HmacSHA1");
			mac.init(skeySpec);
			byte[] temp = mac.doFinal();
			for(int i=0;i<16;i++)
			hashedKey[i]=temp[i];
		}catch(Exception e)
		{
			System.err.println(e.toString());
		}
		return hashedKey;
	}
}
